// Application class implementation
#include "pch.h"
#include "Application.h"
#include "resources.h"

// Define window message to update the timer display
#define WM_UPDATEAPP WM_USER + 1

// Enable visual styles
#pragma comment(linker, "\"/manifestdependency:type='win32' \
name='Microsoft.Windows.Common-Controls' version='6.0.0.0' \
processorArchitecture='*' publicKeyToken='6595b64144ccf1df' language='*'\"")

namespace mainspring {
    Application::Application(const char* appName, const char* appVersion)
        : m_appName(appName)
        , m_appVersion(appVersion)
        , m_timing(false)
        , m_startpauseButtonPressed(false)
        , m_elapsedSaved()
    {
        std::cout << std::format("{} {}\n", m_appName, m_appVersion);
    }

    void Application::run() {
        // Read last time here

        HWND dlg = CreateDialogParam(nullptr, MAKEINTRESOURCE(IDD_DIALOGMAIN), nullptr, reinterpret_cast<DLGPROC>(mainDlgProc), reinterpret_cast<LPARAM>(this));

        HACCEL hAccel = LoadAccelerators(GetModuleHandle(NULL), MAKEINTRESOURCE(IDA_ACCELMAIN));
        MSG msg = {};
        ShowWindow(dlg, SW_SHOW);
        while (GetMessage(&msg, nullptr, 0, 0))
            if (!TranslateAccelerator(dlg, hAccel, &msg))
                IsDialogMessage(dlg, &msg);
    }

    BOOL CALLBACK Application::mainDlgProc(HWND dlg, UINT msg, WPARAM wParam, LPARAM lParam) {
        static Application* app = nullptr;

        switch (msg) {
            case WM_INITDIALOG: {
                app = reinterpret_cast<Application*>(lParam);

                SetWindowText(dlg, app->m_appName);
                SendMessage(dlg, WM_SETICON, ICON_BIG, reinterpret_cast<LPARAM>(LoadIcon(GetModuleHandle(NULL), MAKEINTRESOURCE(IDI_ICONMAIN))));

                POINT cursor = {};
                GetCursorPos(&cursor);
                MONITORINFO mi = {};
                mi.cbSize = sizeof(mi);
                GetMonitorInfo(MonitorFromPoint(cursor, MONITOR_DEFAULTTONEAREST), &mi);
                SetWindowPos(dlg, nullptr, mi.rcMonitor.left + 100, mi.rcMonitor.top + 100, 0, 0, SWP_NOZORDER | SWP_NOSIZE);

                SetDlgItemText(dlg, IDC_STATICTITLE, app->m_appName);
                LOGFONT lFont = {};
                lFont.lfHeight = 30;
                HFONT titleFont = CreateFontIndirect(&lFont);
                SendMessage(GetDlgItem(dlg, IDC_STATICTITLE), WM_SETFONT, reinterpret_cast<WPARAM>(titleFont), NULL);

                std::memcpy(lFont.lfFaceName, "Consolas", 9);
                lFont.lfHeight = 60;
                HFONT timeFont = CreateFontIndirect(&lFont);
                SendMessage(GetDlgItem(dlg, IDC_STATICTIME), WM_SETFONT, reinterpret_cast<WPARAM>(timeFont), NULL);

                SendMessage(dlg, WM_UPDATEAPP, NULL, NULL);
                SetTimer(dlg, 1, 50, nullptr);

                return TRUE;
            }

            case WM_COMMAND:
                switch (LOWORD(wParam)) {
                    case IDC_BUTTONSTARTPAUSE:
                        app->m_startpauseButtonPressed = true;
                        return TRUE;

                    case ID_FILE_NEW: {
                        std::cout << std::format("New!\n");
                        return TRUE;
                    }

                    case ID_FILE_SAVE: {
                        std::cout << std::format("Save!\n");
                        return TRUE;
                    }

                    case ID_FILE_SAVEAS: {
                        std::cout << std::format("Save As!\n");
                        return TRUE;
                    }

                    case ID_FILE_OPEN: {
                        std::cout << std::format("Open!\n");
                        return TRUE;
                    }

                    case ID_FILE_EXIT: {
                        SendMessage(dlg, WM_CLOSE, NULL, NULL);
                        return TRUE;
                    }

                    case ID_HELP_ABOUT:
                        MessageBox(dlg, std::format("{} {}\nCopyright Grayson Riffe 2025\ngraysonriffe.com", app->m_appName, app->m_appVersion).c_str(), std::format("About {}", app->m_appName).c_str(), MB_OK);
                        return TRUE;
                }

                return FALSE;

            case WM_TIMER:
                SendMessage(dlg, WM_UPDATEAPP, NULL, NULL);
                return TRUE;

            case WM_UPDATEAPP: {
                if (app->m_startpauseButtonPressed) {
                    app->m_startpauseButtonPressed = false;

                    if (!app->m_timing)
                        app->m_startTime = Clock::now();
                    else
                        app->m_elapsedSaved += app->getElapsed();

                    app->m_timing = !app->m_timing;
                }

                if (app->m_timing)
                    SetDlgItemText(dlg, IDC_BUTTONSTARTPAUSE, "Pause");
                else
                    SetDlgItemText(dlg, IDC_BUTTONSTARTPAUSE, !app->m_elapsedSaved ? "Start" : "Resume");


                Seconds totalTime = app->m_elapsedSaved + app->getElapsed();

                uint32_t hr = totalTime / (3600), min = totalTime % (3600) / 60, sec = totalTime % 60;
                SetDlgItemText(dlg, IDC_STATICTIME, std::format("{:02}:{:02}:{:02}", hr, min, sec).c_str());

                return TRUE;
            }

            case WM_CLOSE:
                // TODO: Ask to / save time here
                KillTimer(dlg, 1);
                DestroyWindow(dlg);
                return TRUE;

            case WM_DESTROY:
                PostQuitMessage(0);
                return TRUE;
        }

        return FALSE;
    }

    Seconds Application::getElapsed() {
        if (m_timing)
            return Duration(Clock::now() - m_startTime).count();
        else
            return 0;
    }
}